@@ -2,7 +2,7 @@
 
 /*  vcfcall.c -- SNP/indel variant calling from VCF/BCF.
 
-    Copyright (C) 2013-2016 Genome Research Ltd.
+    Copyright (C) 2013-2021 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -27,6 +27,7 @@ THE SOFTWARE.  */
 #include <stdarg.h>
 #include <string.h>
 #include <strings.h>
+#include <assert.h>
 #include <errno.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -44,14 +45,11 @@ THE SOFTWARE.  */
 #include "prob1.h"
 #include "ploidy.h"
 #include "gvcf.h"
+#include "regidx.h"
+#include "vcfbuf.h"
 
 void error(const char *format, ...);
 
-#ifdef _WIN32
-#define srand48(x) srand(x)
-#define lrand48() rand()
-#endif
-
 #define CF_NO_GENO      1
 #define CF_INS_MISSED   (1<<1)
 #define CF_CCALL        (1<<2)
@@ -68,6 +66,13 @@ void error(const char *format, ...);
 #define CF_QCNT         (1<<13)
 #define CF_INDEL_ONLY   (1<<14)
 
+typedef struct
+{
+    tgt_als_t *als;
+    int nmatch_als, ibuf;
+}
+rec_tgt_t;
+
 typedef struct
 {
     int flag;   // combination of CF_* flags above
@@ -78,6 +83,9 @@ typedef struct
     int nsamples, *samples_map; // mapping from output sample names to original VCF
     char *regions, *targets;    // regions to process
     int regions_is_file, targets_is_file;
+    regidx_t *tgt_idx;
+    regitr_t *tgt_itr, *tgt_itr_prev, *tgt_itr_tmp;
+    vcfbuf_t *vcfbuf;
 
     char *samples_fname;
     int samples_is_file;
@@ -88,6 +96,7 @@ typedef struct
 
     bcf1_t *missed_line;
     call_t aux;     // parameters and temporary data
+    kstring_t str;
 
     int argc;
     char **argv;
@@ -183,6 +192,11 @@ static ploidy_predef_t ploidy_predefs[] =
       .ploidy =
           "*  * *     * 1\n"
     },
+    { .alias  = "2",
+      .about  = "Treat all samples as diploid",
+      .ploidy =
+          "*  * *     * 2\n"
+    },
     {
         .alias  = NULL,
         .about  = NULL,
@@ -299,7 +313,7 @@ static void set_samples(args_t *args, const char *fn, int is_file)
         if ( ismpl < 0 ) { fprintf(bcftools_stderr,"Warning: No such sample in the VCF: %s\n",ss); continue; }
         if ( old2new[ismpl] != -1 ) { fprintf(bcftools_stderr,"Warning: The sample is listed multiple times: %s\n",ss); continue; }
 
-        ss = se+1;
+        ss = se+(x != '\0');
         while ( *ss && isspace(*ss) ) ss++;
         if ( !*ss ) ss = "2";   // default ploidy
         se = ss;
@@ -349,26 +363,253 @@ static void init_missed_line(args_t *args)
     bcf_float_set_missing(args->missed_line->qual);
 }
 
-static void print_missed_line(bcf_sr_regions_t *regs, void *data)
+static int tgt_parse(const char *line, char **chr_beg, char **chr_end, uint32_t *beg, uint32_t *end, void *payload, void *usr)
 {
-    args_t *args = (args_t*) data;
-    call_t *call = &args->aux;
-    bcf1_t *missed = args->missed_line;
+    char *ss = (char*) line;
+    while ( *ss && isspace(*ss) ) ss++;
+    if ( !*ss ) { fprintf(bcftools_stderr,"Could not parse the line: %s\n", line); return -2; }
+    if ( *ss=='#' ) return -1;  // skip comments
 
-    char *ss = regs->line.s;
-    int i = 0;
-    while ( i<args->aux.srs->targets_als-1 && *ss )
+    char *se = ss;
+    while ( *se && !isspace(*se) ) se++;
+
+    *chr_beg = ss;
+    *chr_end = se-1;
+
+    if ( !*se ) { fprintf(bcftools_stderr,"Could not parse the line: %s\n", line); return -2; }
+
+    ss = se+1;
+    *beg = strtod(ss, &se);
+    if ( ss==se ) { fprintf(bcftools_stderr,"Could not parse tab line: %s\n", line); return -2; }
+    if ( *beg==0 ) { fprintf(bcftools_stderr,"Could not parse tab line, expected 1-based coordinate: %s\n", line); return -2; }
+    (*beg)--;
+    *end = *beg;
+
+    if ( !usr ) return 0; // allele information not required
+
+    ss = se+1;
+    tgt_als_t *als = (tgt_als_t*)payload;
+    als->used   = 0;
+    als->n      = 0;
+    als->allele = NULL;
+    while ( *ss )
     {
-        if ( *ss=='\t' ) i++;
-        ss++;
+        se = ss;
+        while ( *se && *se!=',' ) se++;
+        als->n++;
+        als->allele = (char**)realloc(als->allele,als->n*sizeof(*als->allele));
+        als->allele[als->n-1] = (char*)malloc(se-ss+1);
+        memcpy(als->allele[als->n-1],ss,se-ss);
+        als->allele[als->n-1][se-ss] = 0;
+        ss = se+1;
+        if ( !*se ) break;
     }
-    if ( !*ss ) error("Could not parse: [%s] (%d)\n", regs->line.s,args->aux.srs->targets_als);
+    return 0;
+}
+static void tgt_free(void *payload)
+{
+    tgt_als_t *als = (tgt_als_t*)payload;
+    int i;
+    for (i=0; i<als->n; i++) free(als->allele[i]);
+    free(als->allele);
+}
+static void tgt_flush_region(args_t *args, char *chr, uint32_t beg, uint32_t end)
+{
+    if ( !regidx_overlap(args->tgt_idx, chr,beg,end,args->tgt_itr_tmp) ) return;
+    while ( regitr_overlap(args->tgt_itr_tmp) )
+    {
+        if ( args->tgt_itr_tmp->beg < beg ) continue;
 
-    missed->rid  = bcf_hdr_name2id(call->hdr,regs->seq_names[regs->prev_seq]);
-    missed->pos  = regs->start;
-    bcf_update_alleles_str(call->hdr, missed,ss);
+        tgt_als_t *tgt_als = &regitr_payload(args->tgt_itr_tmp,tgt_als_t);
+        if ( tgt_als->used ) continue;
 
-    bcf_write1(args->out_fh, call->hdr, missed);
+        args->missed_line->rid  = bcf_hdr_name2id(args->aux.hdr,chr);
+        args->missed_line->pos  = args->tgt_itr_tmp->beg;
+        bcf_unpack(args->missed_line,BCF_UN_ALL);
+        bcf_update_alleles(args->aux.hdr, args->missed_line, (const char**)tgt_als->allele, tgt_als->n);
+        tgt_als->used = 1;
+        if ( bcf_write1(args->out_fh, args->aux.hdr, args->missed_line)!=0 ) error("[%s] Error: failed to write to %s\n", __func__,args->output_fname);
+    }
+}
+static void tgt_flush(args_t *args, bcf1_t *rec)
+{
+    if ( rec )
+    {
+        char *chr = (char*)bcf_seqname(args->aux.hdr,rec);
+
+        if ( !args->tgt_itr_prev )                  // first record
+            tgt_flush_region(args,chr,0,rec->pos-1);
+
+        else if ( strcmp(chr,args->tgt_itr_prev->seq) )  // first record on a new chromosome
+        {
+            tgt_flush_region(args,args->tgt_itr_prev->seq,args->tgt_itr_prev->beg+1,REGIDX_MAX);
+            tgt_flush_region(args,chr,0,rec->pos-1);
+        }
+        else                                        // another record on the same chromosome
+            tgt_flush_region(args,args->tgt_itr_prev->seq,args->tgt_itr_prev->beg,rec->pos-1);
+    }
+    else
+    {
+        // flush everything
+        if ( args->tgt_itr_prev )
+            tgt_flush_region(args,args->tgt_itr_prev->seq,args->tgt_itr_prev->beg,REGIDX_MAX);
+
+        int i, nchr = 0;
+        char **chr = regidx_seq_names(args->tgt_idx, &nchr);
+        for (i=0; i<nchr; i++)
+            tgt_flush_region(args,chr[i],0,REGIDX_MAX);
+    }
+}
+inline static int is_indel(int nals, char **als)
+{
+    // This is mpileup output, we can make some assumption:
+    //  - no MNPs
+    //  - "<*>" is not present at indels sites and there are no other symbolic alleles than <*>
+    if ( als[1][0]=='<' ) return 0;
+
+    int i;
+    for (i=0; i<nals; i++)
+    {
+        if ( als[i][0]=='<' ) continue;
+        if ( als[i][1] ) return 1;
+    }
+    return 0;
+}
+bcf1_t *next_line(args_t *args)
+{
+    bcf1_t *rec = NULL;
+    if ( !args->vcfbuf )
+    {
+        while ( bcf_sr_next_line(args->aux.srs) )
+        {
+            rec = args->aux.srs->readers[0].buffer[0];
+            if ( args->aux.srs->errnum || rec->errcode ) error("Error: could not parse the input VCF\n");
+            if ( args->tgt_idx )
+            {
+                if ( !regidx_overlap(args->tgt_idx, bcf_seqname(args->aux.hdr,rec),rec->pos,rec->pos,args->tgt_itr) ) continue;
+
+                // For backward compatibility: require the exact position, not an interval overlap
+                int pos_match = 0;
+                while ( regitr_overlap(args->tgt_itr) )
+                {
+                    if ( args->tgt_itr->beg != rec->pos ) continue;
+                    pos_match = 1;
+                    break;
+                }
+                if ( !pos_match ) continue;
+            }
+            if ( args->samples_map ) bcf_subset(args->aux.hdr, rec, args->nsamples, args->samples_map);
+            bcf_unpack(rec, BCF_UN_STR);
+            return rec;
+        }
+        return NULL;
+    }
+
+    // If we are here,-C alleles was given and vcfbuf and tgt_idx are set
+
+    // Fill the buffer with duplicate lines
+    int vcfbuf_full = 1;
+    int nbuf = vcfbuf_nsites(args->vcfbuf);
+    bcf1_t *rec0 = NULL, *recN = NULL;
+    if ( nbuf==0 ) vcfbuf_full = 0;
+    else if ( nbuf==1 )
+    {
+        vcfbuf_full = 0;
+        rec0 = vcfbuf_peek(args->vcfbuf, 0);
+    }
+    else
+    {
+        rec0 = vcfbuf_peek(args->vcfbuf, 0);
+        recN = vcfbuf_peek(args->vcfbuf, nbuf-1);
+        if ( rec0->rid == recN->rid && rec0->pos == recN->pos ) vcfbuf_full = 0;
+    }
+    if ( !vcfbuf_full )
+    {
+        while ( bcf_sr_next_line(args->aux.srs) )
+        {
+            rec = args->aux.srs->readers[0].buffer[0];
+            if ( args->aux.srs->errnum || rec->errcode ) error("Error: could not parse the input VCF\n");
+            if ( !regidx_overlap(args->tgt_idx, bcf_seqname(args->aux.hdr,rec),rec->pos,rec->pos,args->tgt_itr) ) continue;
+            // as above: require the exact position, not an interval overlap
+            int exact_match = 0;
+            while ( regitr_overlap(args->tgt_itr) )
+            {
+                if ( args->tgt_itr->beg != rec->pos ) continue;
+                exact_match = 1;
+                break;
+            }
+            if ( !exact_match ) continue;
+
+            if ( args->samples_map ) bcf_subset(args->aux.hdr, rec, args->nsamples, args->samples_map);
+            bcf_unpack(rec, BCF_UN_STR);
+            if ( !rec0 ) rec0 = rec;
+            recN = rec;
+            args->aux.srs->readers[0].buffer[0] = vcfbuf_push(args->vcfbuf, rec);
+            if ( rec0->rid!=recN->rid || rec0->pos!=recN->pos ) break;
+        }
+    }
+
+    nbuf = vcfbuf_nsites(args->vcfbuf);
+    int n, i,j;
+    for (n=nbuf; n>1; n--)
+    {
+        recN = vcfbuf_peek(args->vcfbuf, n-1);
+        if ( rec0->rid==recN->rid && rec0->pos==recN->pos ) break;
+    }
+    if ( n==0 )
+    {
+        assert( !nbuf );
+        return NULL;
+    }
+
+    // Find the VCF and tab record with the best matching combination of alleles, prioritize 
+    // records of the same type (snp vs indel)
+    rec_tgt_t rec_tgt;
+    memset(&rec_tgt,0,sizeof(rec_tgt));
+    regidx_overlap(args->tgt_idx, bcf_seqname(args->aux.hdr,rec0),rec0->pos,rec0->pos,args->tgt_itr);
+    regitr_t *tmp_itr = regitr_init(args->tgt_idx);
+    regitr_copy(tmp_itr, args->tgt_itr);
+    for (i=0; i<n; i++)
+    {
+        rec = vcfbuf_peek(args->vcfbuf, i);
+        int rec_indel = is_indel(rec->n_allele, rec->d.allele) ? 1 : -1;
+        while ( regitr_overlap(tmp_itr) )
+        {
+            if ( tmp_itr->beg != rec->pos ) continue;
+            tgt_als_t *als = &regitr_payload(tmp_itr,tgt_als_t);
+            if ( als->used ) continue;
+            int nmatch_als = 0;
+            vcmp_t *vcmp = vcmp_init();
+            int ret = vcmp_set_ref(vcmp, rec->d.allele[0], als->allele[0]);
+            if ( ret==0 )
+            {
+                nmatch_als++;
+                if ( rec->n_allele > 1 && als->n > 1 )
+                {
+                    for (j=1; j<als->n; j++)
+                    {
+                        if ( vcmp_find_allele(vcmp, rec->d.allele+1, rec->n_allele-1, als->allele[j])>=0 ) nmatch_als++;
+                    }
+                }
+            }
+            int als_indel = is_indel(als->n, als->allele) ? 1 : -1;
+            nmatch_als *= rec_indel*als_indel;
+            if ( nmatch_als > rec_tgt.nmatch_als || !rec_tgt.als )
+            {
+                rec_tgt.nmatch_als = nmatch_als;
+                rec_tgt.als  = als;
+                rec_tgt.ibuf = i;
+            }
+            vcmp_destroy(vcmp);
+        }
+    }
+    regitr_destroy(tmp_itr);
+
+    args->aux.tgt_als = rec_tgt.als;
+    if ( rec_tgt.als ) rec_tgt.als->used = 1;
+
+    rec = vcfbuf_remove(args->vcfbuf, rec_tgt.ibuf);
+    return rec;
 }
 
 static void init_data(args_t *args)
@@ -378,22 +619,19 @@ static void init_data(args_t *args)
     // Open files for input and output, initialize structures
     if ( args->targets )
     {
-        if ( bcf_sr_set_targets(args->aux.srs, args->targets, args->targets_is_file, args->aux.flag&CALL_CONSTR_ALLELES ? 3 : 0)<0 )
-            error("Failed to read the targets: %s\n", args->targets);
-
-        if ( args->aux.flag&CALL_CONSTR_ALLELES && args->flag&CF_INS_MISSED )
-        {
-            args->aux.srs->targets->missed_reg_handler = print_missed_line;
-            args->aux.srs->targets->missed_reg_data = args;
-        }
+        args->tgt_idx = regidx_init(args->targets, tgt_parse, args->aux.flag&CALL_CONSTR_ALLELES ? tgt_free : (regidx_free_f) NULL, sizeof(tgt_als_t), args->aux.flag&CALL_CONSTR_ALLELES ? args : NULL);
+        args->tgt_itr = regitr_init(args->tgt_idx);
+        args->tgt_itr_tmp = regitr_init(args->tgt_idx);
     }
+
     if ( args->regions )
     {
         if ( bcf_sr_set_regions(args->aux.srs, args->regions, args->regions_is_file)<0 )
             error("Failed to read the regions: %s\n", args->regions);
     }
 
-    if ( !bcf_sr_add_reader(args->aux.srs, args->bcf_fname) ) error("Failed to open %s: %s\n", args->bcf_fname,bcf_sr_strerror(args->aux.srs->errnum));
+    if ( !bcf_sr_add_reader(args->aux.srs, args->bcf_fname) )
+        error("Failed to read from %s: %s\n", !strcmp("-",args->bcf_fname)?"standard input":args->bcf_fname,bcf_sr_strerror(args->aux.srs->errnum));
     args->aux.hdr = bcf_sr_get_header(args->aux.srs,0);
 
     int i;
@@ -453,8 +691,11 @@ static void init_data(args_t *args)
         }
     }
 
-    args->out_fh = hts_open(args->output_fname, hts_bcf_wmode(args->output_type));
-    if ( args->out_fh == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
+    if ( args->aux.flag & CALL_CONSTR_ALLELES )
+        args->vcfbuf = vcfbuf_init(args->aux.hdr, 0);
+
+    args->out_fh = hts_open(args->output_fname, hts_bcf_wmode2(args->output_type,args->output_fname));
+    if ( args->out_fh == NULL ) error("Error: cannot write to \"%s\": %s\n", args->output_fname, strerror(errno));
     if ( args->n_threads ) hts_set_threads(args->out_fh, args->n_threads);
 
     if ( args->flag & CF_QCALL )
@@ -470,13 +711,21 @@ static void init_data(args_t *args)
     bcf_hdr_remove(args->aux.hdr, BCF_HL_INFO, "I16");
 
     if (args->record_cmd_line) bcf_hdr_append_version(args->aux.hdr, args->argc, args->argv, "bcftools_call");
-    bcf_hdr_write(args->out_fh, args->aux.hdr);
+    if ( bcf_hdr_write(args->out_fh, args->aux.hdr)!=0 ) error("[%s] Error: cannot write the header to %s\n", __func__,args->output_fname);
 
     if ( args->flag&CF_INS_MISSED ) init_missed_line(args);
 }
 
 static void destroy_data(args_t *args)
 {
+    if ( args->vcfbuf ) vcfbuf_destroy(args->vcfbuf);
+    if ( args->tgt_idx )
+    {
+        regidx_destroy(args->tgt_idx);
+        regitr_destroy(args->tgt_itr);
+        regitr_destroy(args->tgt_itr_tmp);
+        if ( args->tgt_itr_prev ) regitr_destroy(args->tgt_itr_prev);
+    }
     if ( args->flag & CF_CCALL ) ccall_destroy(&args->aux);
     else if ( args->flag & CF_MCALL ) mcall_destroy(&args->aux);
     else if ( args->flag & CF_QCALL ) qcall_destroy(&args->aux);
@@ -498,9 +747,10 @@ static void destroy_data(args_t *args)
     free(args->samples_map);
     free(args->sample2sex);
     free(args->aux.ploidy);
+    free(args->str.s);
     if ( args->gvcf ) gvcf_destroy(args->gvcf);
     bcf_hdr_destroy(args->aux.hdr);
-    hts_close(args->out_fh);
+    if ( hts_close(args->out_fh)!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->output_fname);
     bcf_sr_destroy(args->aux.srs);
 }
 
@@ -526,7 +776,20 @@ void parse_novel_rate(args_t *args, const char *str)
     else error("Could not parse --novel-rate %s\n", str);
 }
 
-static int parse_format_flag(const char *str)
+static void list_annotations(FILE *fp)
+{
+    fprintf(fp,
+        "\n"
+        "Optional INFO annotations available with -m (\"INFO/\" prefix is optional):\n"
+        "  INFO/PV4   .. P-values for strand bias, baseQ bias, mapQ bias and tail distance bias (Number=4,Type=Float)\n"
+        "\n"
+        "Optional FORMAT annotations available with -m (\"FORMAT/\" prefix is optional):\n"
+        "  FORMAT/GQ  .. Phred-scaled genotype quality (Number=1,Type=Integer)\n"
+        "  FORMAT/GP  .. Phred-scaled genotype posterior probabilities (Number=G,Type=Float)\n"
+        "\n");
+}
+
+static int parse_output_tags(const char *str)
 {
     int flag = 0;
     const char *ss = str;
@@ -534,12 +797,13 @@ static int parse_format_flag(const char *str)
     {
         const char *se = ss;
         while ( *se && *se!=',' ) se++;
-        if ( !strncasecmp(ss,"GQ",se-ss) ) flag |= CALL_FMT_GQ;
-        else if ( !strncasecmp(ss,"GP",se-ss) ) flag |= CALL_FMT_GP;
+        if ( !strncasecmp(ss,"GQ",se-ss) || !strncasecmp(ss,"FORMAT/GQ",se-ss) || !strncasecmp(ss,"FMT/GQ",se-ss)  ) flag |= CALL_FMT_GQ;
+        else if ( !strncasecmp(ss,"GP",se-ss) || !strncasecmp(ss,"FORMAT/GP",se-ss) || !strncasecmp(ss,"FMT/GP",se-ss) ) flag |= CALL_FMT_GP;
+        else if ( !strncasecmp(ss,"PV4",se-ss) || !strncasecmp(ss,"INFO/PV4",se-ss) ) flag |= CALL_FMT_PV4;
         else
         {
             fprintf(bcftools_stderr,"Could not parse \"%s\"\n", str);
-            exit(1);
+            bcftools_exit(1);
         }
         if ( !*se ) break;
         ss = se + 1;
@@ -593,12 +857,12 @@ ploidy_t *init_ploidy(char *alias)
         fprintf(bcftools_stderr,"Run as --ploidy <alias> (e.g. --ploidy GRCh37).\n");
         fprintf(bcftools_stderr,"To see the detailed ploidy definition, append a question mark (e.g. --ploidy GRCh37?).\n");
         fprintf(bcftools_stderr,"\n");
-        exit(-1);
+        bcftools_exit(-1);
     }
     else if ( detailed )
     {
         fprintf(bcftools_stderr,"%s", pld->ploidy);
-        exit(-1);
+        bcftools_exit(-1);
     }
     return ploidy_init_string(pld->ploidy,2);
 }
@@ -606,7 +870,7 @@ ploidy_t *init_ploidy(char *alias)
 static void usage(args_t *args)
 {
     fprintf(bcftools_stderr, "\n");
-    fprintf(bcftools_stderr, "About:   SNP/indel variant calling from VCF/BCF. To be used in conjunction with samtools mpileup.\n");
+    fprintf(bcftools_stderr, "About:   SNP/indel variant calling from VCF/BCF. To be used in conjunction with bcftools mpileup.\n");
     fprintf(bcftools_stderr, "         This command replaces the former \"bcftools view\" caller. Some of the original\n");
     fprintf(bcftools_stderr, "         functionality has been temporarily lost in the process of transition to htslib,\n");
     fprintf(bcftools_stderr, "         but will be added back on popular demand. The original calling model can be\n");
@@ -614,36 +878,46 @@ static void usage(args_t *args)
     fprintf(bcftools_stderr, "Usage:   bcftools call [options] <in.vcf.gz>\n");
     fprintf(bcftools_stderr, "\n");
     fprintf(bcftools_stderr, "File format options:\n");
-    fprintf(bcftools_stderr, "       --no-version                do not append version and command line to the header\n");
-    fprintf(bcftools_stderr, "   -o, --output <file>             write output to a file [standard output]\n");
-    fprintf(bcftools_stderr, "   -O, --output-type <b|u|z|v>     output type: 'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
-    fprintf(bcftools_stderr, "       --ploidy <assembly>[?]      predefined ploidy, 'list' to print available settings, append '?' for details\n");
-    fprintf(bcftools_stderr, "       --ploidy-file <file>        space/tab-delimited list of CHROM,FROM,TO,SEX,PLOIDY\n");
-    fprintf(bcftools_stderr, "   -r, --regions <region>          restrict to comma-separated list of regions\n");
-    fprintf(bcftools_stderr, "   -R, --regions-file <file>       restrict to regions listed in a file\n");
-    fprintf(bcftools_stderr, "   -s, --samples <list>            list of samples to include [all samples]\n");
-    fprintf(bcftools_stderr, "   -S, --samples-file <file>       PED file or a file with an optional column with sex (see man page for details) [all samples]\n");
-    fprintf(bcftools_stderr, "   -t, --targets <region>          similar to -r but streams rather than index-jumps\n");
-    fprintf(bcftools_stderr, "   -T, --targets-file <file>       similar to -R but streams rather than index-jumps\n");
-    fprintf(bcftools_stderr, "       --threads <int>             number of extra output compression threads [0]\n");
+    fprintf(bcftools_stderr, "       --no-version              Do not append version and command line to the header\n");
+    fprintf(bcftools_stderr, "   -o, --output FILE             Write output to a file [standard output]\n");
+    fprintf(bcftools_stderr, "   -O, --output-type b|u|z|v     Output type: 'b' compressed BCF; 'u' uncompressed BCF; 'z' compressed VCF; 'v' uncompressed VCF [v]\n");
+    fprintf(bcftools_stderr, "       --ploidy ASSEMBLY[?]      Predefined ploidy, 'list' to print available settings, append '?' for details [2]\n");
+    fprintf(bcftools_stderr, "       --ploidy-file FILE        Space/tab-delimited list of CHROM,FROM,TO,SEX,PLOIDY\n");
+    fprintf(bcftools_stderr, "   -r, --regions REGION          Restrict to comma-separated list of regions\n");
+    fprintf(bcftools_stderr, "   -R, --regions-file FILE       Restrict to regions listed in a file\n");
+    fprintf(bcftools_stderr, "   -s, --samples LIST            List of samples to include [all samples]\n");
+    fprintf(bcftools_stderr, "   -S, --samples-file FILE       PED file or a file with an optional column with sex (see man page for details) [all samples]\n");
+    fprintf(bcftools_stderr, "   -t, --targets REGION          Similar to -r but streams rather than index-jumps\n");
+    fprintf(bcftools_stderr, "   -T, --targets-file FILE       Similar to -R but streams rather than index-jumps\n");
+    fprintf(bcftools_stderr, "       --threads INT             Use multithreading with INT worker threads [0]\n");
     fprintf(bcftools_stderr, "\n");
     fprintf(bcftools_stderr, "Input/output options:\n");
-    fprintf(bcftools_stderr, "   -A, --keep-alts                 keep all possible alternate alleles at variant sites\n");
-    fprintf(bcftools_stderr, "   -f, --format-fields <list>      output format fields: GQ,GP (lowercase allowed) []\n");
-    fprintf(bcftools_stderr, "   -F, --prior-freqs <AN,AC>       use prior allele frequencies\n");
-    fprintf(bcftools_stderr, "   -g, --gvcf <int>,[...]          group non-variant sites into gVCF blocks by minimum per-sample DP\n");
-    fprintf(bcftools_stderr, "   -i, --insert-missed             output also sites missed by mpileup but present in -T\n");
-    fprintf(bcftools_stderr, "   -M, --keep-masked-ref           keep sites with masked reference allele (REF=N)\n");
-    fprintf(bcftools_stderr, "   -V, --skip-variants <type>      skip indels/snps\n");
-    fprintf(bcftools_stderr, "   -v, --variants-only             output variant sites only\n");
+    fprintf(bcftools_stderr, "   -A, --keep-alts               Keep all possible alternate alleles at variant sites\n");
+    fprintf(bcftools_stderr, "   -a, --annotate LIST           Optional tags to output (lowercase allowed); '?' to list available tags\n");
+//todo?    
+//    fprintf(bcftools_stderr, "   -a, --annots LIST             Add annotations: GQ,GP,PV4 (lowercase allowed). Prefixed with ^ indicates a request for\n");
+//    fprintf(bcftools_stderr, "                                 tag removal [^I16,^QS,^FMT/QS]\n");
+    fprintf(bcftools_stderr, "   -F, --prior-freqs AN,AC       Use prior allele frequencies, determined from these pre-filled tags\n");
+    fprintf(bcftools_stderr, "   -G, --group-samples FILE|-    Group samples by population (file with \"sample\\tgroup\") or \"-\" for single-sample calling.\n");
+    fprintf(bcftools_stderr, "                                 This requires FORMAT/QS or other Number=R,Type=Integer tag such as FORMAT/AD\n"); 
+    fprintf(bcftools_stderr, "       --group-samples-tag TAG   The tag to use with -G, by default FORMAT/QS and FORMAT/AD are checked automatically\n");
+    fprintf(bcftools_stderr, "   -g, --gvcf INT,[...]          Group non-variant sites into gVCF blocks by minimum per-sample DP\n");
+    fprintf(bcftools_stderr, "   -i, --insert-missed           Output also sites missed by mpileup but present in -T\n");
+    fprintf(bcftools_stderr, "   -M, --keep-masked-ref         Keep sites with masked reference allele (REF=N)\n");
+    fprintf(bcftools_stderr, "   -V, --skip-variants TYPE      Skip indels/snps\n");
+    fprintf(bcftools_stderr, "   -v, --variants-only           Output variant sites only\n");
     fprintf(bcftools_stderr, "\n");
     fprintf(bcftools_stderr, "Consensus/variant calling options:\n");
-    fprintf(bcftools_stderr, "   -c, --consensus-caller          the original calling method (conflicts with -m)\n");
-    fprintf(bcftools_stderr, "   -C, --constrain <str>           one of: alleles, trio (see manual)\n");
-    fprintf(bcftools_stderr, "   -m, --multiallelic-caller       alternative model for multiallelic and rare-variant calling (conflicts with -c)\n");
-    fprintf(bcftools_stderr, "   -n, --novel-rate <float>,[...]  likelihood of novel mutation for constrained trio calling, see man page for details [1e-8,1e-9,1e-9]\n");
-    fprintf(bcftools_stderr, "   -p, --pval-threshold <float>    variant if P(ref|D)<FLOAT with -c [0.5]\n");
-    fprintf(bcftools_stderr, "   -P, --prior <float>             mutation rate (use bigger for greater sensitivity), use with -m [1.1e-3]\n");
+    fprintf(bcftools_stderr, "   -c, --consensus-caller        The original calling method (conflicts with -m)\n");
+    fprintf(bcftools_stderr, "   -C, --constrain STR           One of: alleles, trio (see manual)\n");
+    fprintf(bcftools_stderr, "   -m, --multiallelic-caller     Alternative model for multiallelic and rare-variant calling (conflicts with -c)\n");
+    fprintf(bcftools_stderr, "   -n, --novel-rate FLOAT,[...]  Likelihood of novel mutation for constrained trio calling, see man page for details [1e-8,1e-9,1e-9]\n");
+    fprintf(bcftools_stderr, "   -p, --pval-threshold FLOAT    Variant if P(ref|D)<FLOAT with -c [0.5]\n");
+    fprintf(bcftools_stderr, "   -P, --prior FLOAT             Mutation rate (use bigger for greater sensitivity), use with -m [1.1e-3]\n");
+    fprintf(bcftools_stderr, "\n");
+    fprintf(bcftools_stderr, "Example:\n");
+    fprintf(bcftools_stderr, "   # See also http://samtools.github.io/bcftools/howtos/variant-calling.html\n");
+    fprintf(bcftools_stderr, "   bcftools mpileup -Ou -f reference.fa alignments.bam | bcftools call -mv -Ob -o calls.bcf\n");
 
     // todo (and more)
     // fprintf(bcftools_stderr, "\nContrast calling and association test options:\n");
@@ -652,7 +926,7 @@ static void usage(args_t *args)
     // fprintf(bcftools_stderr, "       -U INT    number of permutations for association testing (effective with -1) [0]\n");
     // fprintf(bcftools_stderr, "       -X FLOAT  only perform permutations for P(chi^2)<FLOAT [%g]\n", args->aux.min_perm_p);
     fprintf(bcftools_stderr, "\n");
-    exit(-1);
+    bcftools_exit(-1);
 }
 
 int main_vcfcall(int argc, char *argv[])
@@ -680,8 +954,11 @@ int main_vcfcall(int argc, char *argv[])
     {
         {"help",no_argument,NULL,'h'},
         {"format-fields",required_argument,NULL,'f'},
+        {"annotate",required_argument,NULL,'a'},
         {"prior-freqs",required_argument,NULL,'F'},
         {"gvcf",required_argument,NULL,'g'},
+        {"group-samples",required_argument,NULL,'G'},
+        {"group-samples-tag",required_argument,NULL,3},
         {"output",required_argument,NULL,'o'},
         {"output-type",required_argument,NULL,'O'},
         {"regions",required_argument,NULL,'r'},
@@ -712,7 +989,7 @@ int main_vcfcall(int argc, char *argv[])
     };
 
     char *tmp = NULL;
-    while ((c = getopt_long(argc, argv, "h?o:O:r:R:s:S:t:T:ANMV:vcmp:C:n:P:f:ig:XYF:", loptions, NULL)) >= 0)
+    while ((c = getopt_long(argc, argv, "h?o:O:r:R:s:S:t:T:ANMV:vcmp:C:n:P:f:a:ig:XYF:G:", loptions, NULL)) >= 0)
     {
         switch (c)
         {
@@ -720,7 +997,13 @@ int main_vcfcall(int argc, char *argv[])
             case  1 : ploidy = optarg; break;
             case 'X': ploidy = "X"; fprintf(bcftools_stderr,"Warning: -X will be deprecated, please use --ploidy instead.\n"); break;
             case 'Y': ploidy = "Y"; fprintf(bcftools_stderr,"Warning: -Y will be deprecated, please use --ploidy instead.\n"); break;
-            case 'f': args.aux.output_tags |= parse_format_flag(optarg); break;
+            case 'G': args.aux.sample_groups = optarg; break;
+            case  3 : args.aux.sample_groups_tag = optarg; break;
+            case 'f': fprintf(bcftools_stderr,"Warning: -f, --format-fields will be deprecated, please use -a, --annotate instead.\n");
+            case 'a':
+                      if (optarg[0]=='?') { list_annotations(bcftools_stderr); return 1; }
+                      args.aux.output_tags |= parse_output_tags(optarg);
+                      break;
             case 'M': args.flag &= ~CF_ACGT_ONLY; break;     // keep sites where REF is N
             case 'N': args.flag |= CF_ACGT_ONLY; break;      // omit sites where first base in REF is N (the new default)
             case 'A': args.aux.flag |= CALL_KEEPALT; break;
@@ -807,13 +1090,14 @@ int main_vcfcall(int argc, char *argv[])
     }
     if ( args.flag & CF_INS_MISSED && !(args.aux.flag&CALL_CONSTR_ALLELES) ) error("The -i option requires -C alleles\n");
     if ( args.aux.flag&CALL_VARONLY && args.gvcf ) error("The two options cannot be combined: --variants-only and --gvcf\n");
+    if ( args.aux.sample_groups && !(args.flag & CF_MCALL) ) error("The -G feature is supported only with the -m calling mode\n");
     init_data(&args);
 
-    while ( bcf_sr_next_line(args.aux.srs) )
+    bcf1_t *bcf_rec;
+    while ( (bcf_rec = next_line(&args)) )
     {
-        bcf1_t *bcf_rec = args.aux.srs->readers[0].buffer[0];
-        if ( args.samples_map ) bcf_subset(args.aux.hdr, bcf_rec, args.nsamples, args.samples_map);
-        bcf_unpack(bcf_rec, BCF_UN_STR);
+        // Skip duplicate positions with all matching `-C alleles -T` used up
+        if ( args.aux.flag&CALL_CONSTR_ALLELES && !args.aux.tgt_als ) continue;
 
         // Skip unwanted sites
         int i, is_indel = bcf_is_snp(bcf_rec) ? 0 : 1;
@@ -847,6 +1131,13 @@ int main_vcfcall(int argc, char *argv[])
             continue;
         }
 
+        if ( args.flag & CF_INS_MISSED )
+        {
+            tgt_flush(&args,bcf_rec);
+            if ( !args.tgt_itr_prev ) args.tgt_itr_prev = regitr_init(args.tgt_idx);
+            regitr_copy(args.tgt_itr_prev, args.tgt_itr);
+        }
+
         // Calling modes which output VCFs
         int ret;
         if ( args.flag & CF_MCALL )
@@ -860,11 +1151,10 @@ int main_vcfcall(int argc, char *argv[])
         if ( (args.aux.flag & CALL_VARONLY) && ret==0 && !args.gvcf ) continue;     // not a variant
         if ( args.gvcf )
             bcf_rec = gvcf_write(args.gvcf, args.out_fh, args.aux.hdr, bcf_rec, ret==1?1:0);
-        if ( bcf_rec )
-            bcf_write1(args.out_fh, args.aux.hdr, bcf_rec);
+        if ( bcf_rec && bcf_write1(args.out_fh, args.aux.hdr, bcf_rec)!=0 ) error("[%s] Error: failed to write to %s\n", __func__,args.output_fname);
     }
     if ( args.gvcf ) gvcf_write(args.gvcf, args.out_fh, args.aux.hdr, NULL, 0);
-    if ( args.flag & CF_INS_MISSED ) bcf_sr_regions_flush(args.aux.srs->targets);
+    if ( args.flag & CF_INS_MISSED ) tgt_flush(&args,NULL);
     destroy_data(&args);
     return 0;
 }
