@@ -1,6 +1,6 @@
 /*  vcfisec.c -- Create intersections, unions and complements of VCF files.
 
-    Copyright (C) 2012-2014 Genome Research Ltd.
+    Copyright (C) 2012-2021 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -33,6 +33,7 @@ THE SOFTWARE.  */
 #include <htslib/vcf.h>
 #include <htslib/synced_bcf_reader.h>
 #include <htslib/vcfutils.h>
+#include <htslib/hts_os.h>
 #include "bcftools.h"
 #include "filter.h"
 
@@ -140,11 +141,11 @@ void isec_vcf(args_t *args)
     if ( args->targets_list && files->nreaders==1 ) out_std = 1;
     if ( out_std )
     {
-        out_fh = hts_open(args->output_fname? args->output_fname : "-",hts_bcf_wmode(args->output_type));
+        out_fh = hts_open(args->output_fname? args->output_fname : "-",hts_bcf_wmode2(args->output_type,args->output_fname));
         if ( out_fh == NULL ) error("Can't write to %s: %s\n", args->output_fname? args->output_fname : "standard output", strerror(errno));
         if ( args->n_threads ) hts_set_threads(out_fh, args->n_threads);
         if (args->record_cmd_line) bcf_hdr_append_version(files->readers[args->iwrite].header,args->argc,args->argv,"bcftools_isec");
-        bcf_hdr_write(out_fh, files->readers[args->iwrite].header);
+        if ( bcf_hdr_write(out_fh, files->readers[args->iwrite].header)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->output_fname?args->output_fname:"standard output");
     }
     if ( !args->nwrite && !out_std && !args->prefix )
         fprintf(stderr,"Note: -w option not given, printing list of sites...\n");
@@ -195,8 +196,8 @@ void isec_vcf(args_t *args)
 
         if ( out_std )
         {
-            if ( bcf_sr_has_line(files,args->iwrite) )
-                bcf_write1(out_fh, files->readers[args->iwrite].header, files->readers[args->iwrite].buffer[0]);
+            if ( bcf_sr_has_line(files,args->iwrite) && bcf_write1(out_fh, files->readers[args->iwrite].header, files->readers[args->iwrite].buffer[0])!=0 )
+                error("[%s] Error: cannot write to %s\n", __func__, args->output_fname ? args->output_fname : "standard output");
             continue;
         }
         else if ( args->fh_sites )
@@ -218,7 +219,8 @@ void isec_vcf(args_t *args)
             for (i=0; i<files->nreaders; i++)
                 kputc(bcf_sr_has_line(files,i)?'1':'0', &str);
             kputc('\n', &str);
-            fwrite(str.s,sizeof(char),str.l,args->fh_sites);
+            if ( fwrite(str.s,sizeof(char),str.l,args->fh_sites)!=str.l )
+                error("[%s] Error: failed to write %d bytes to %s\n", __func__,(int)str.l,args->output_fname ? args->output_fname : "standard output");
         }
 
         if ( args->prefix )
@@ -226,9 +228,15 @@ void isec_vcf(args_t *args)
             if ( args->isec_op==OP_VENN && ret==3 )
             {
                 if ( !args->nwrite || args->write[0] )
-                    bcf_write1(args->fh_out[2], bcf_sr_get_header(files,0), bcf_sr_get_line(files,0));
+                {
+                    if ( bcf_write1(args->fh_out[2], bcf_sr_get_header(files,0), bcf_sr_get_line(files,0))!=0 )
+                         error("[%s] Error: cannot write\n", __func__);
+                }
                 if ( !args->nwrite || args->write[1] )
-                    bcf_write1(args->fh_out[3], bcf_sr_get_header(files,1), bcf_sr_get_line(files,1));
+                {
+                    if ( bcf_write1(args->fh_out[3], bcf_sr_get_header(files,1), bcf_sr_get_line(files,1))!=0 )
+                        error("[%s] Error: cannot write\n", __func__);
+                }
             }
             else
             {
@@ -236,13 +244,13 @@ void isec_vcf(args_t *args)
                 {
                     if ( !bcf_sr_has_line(files,i) ) continue;
                     if ( args->write && !args->write[i] ) continue;
-                    bcf_write1(args->fh_out[i], files->readers[i].header, files->readers[i].buffer[0]);
+                    if ( bcf_write1(args->fh_out[i], files->readers[i].header, files->readers[i].buffer[0])!=0 ) error("[%s] Error: cannot write\n", __func__);
                 }
             }
         }
     }
     if ( str.s ) free(str.s);
-    if ( out_fh ) hts_close(out_fh);
+    if ( out_fh && hts_close(out_fh)!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->output_fname? args->output_fname : "-");
 }
 
 static void add_filter(args_t *args, char *expr, int logic)
@@ -348,11 +356,11 @@ static void init_data(args_t *args)
 
             #define OPEN_FILE(i,j) { \
                 open_file(&args->fnames[i], NULL, "%s/%04d.%s", args->prefix, i, suffix); \
-                args->fh_out[i] = hts_open(args->fnames[i], hts_bcf_wmode(args->output_type));  \
+                args->fh_out[i] = hts_open(args->fnames[i], hts_bcf_wmode2(args->output_type,args->fnames[i]));  \
                 if ( !args->fh_out[i] ) error("Could not open %s\n", args->fnames[i]); \
                 if ( args->n_threads ) hts_set_threads(args->fh_out[i], args->n_threads); \
                 if (args->record_cmd_line) bcf_hdr_append_version(args->files->readers[j].header,args->argc,args->argv,"bcftools_isec"); \
-                bcf_hdr_write(args->fh_out[i], args->files->readers[j].header); \
+                if ( bcf_hdr_write(args->fh_out[i], args->files->readers[j].header)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->fnames[i]); \
             }
             if ( !args->nwrite || args->write[0] )
             {
@@ -389,10 +397,9 @@ static void init_data(args_t *args)
                 fprintf(args->fh_log,"%s\tfor stripped\t%s\n", args->fnames[i], args->files->readers[i].fname);
             }
             #undef OPEN_FILE
-
-            args->fh_sites = open_file(NULL, "w", "%s/sites.txt", args->prefix);
-            if ( !args->fh_sites ) error("%s/sites.txt: %s\n", args->prefix, strerror(errno));
         }
+        args->fh_sites = open_file(NULL, "w", "%s/sites.txt", args->prefix);
+        if ( !args->fh_sites ) error("%s/sites.txt: %s\n", args->prefix, strerror(errno));
     }
     else {
         if (args->output_fname) {
@@ -425,7 +432,7 @@ static void destroy_data(args_t *args)
         for (i=0; i<n; i++)
         {
             if ( !args->fnames[i] ) continue;
-            hts_close(args->fh_out[i]);
+            if ( hts_close(args->fh_out[i])!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->fnames[i]);
             if ( args->output_type==FT_VCF_GZ )
             {
                 tbx_conf_t conf = tbx_conf_vcf;
@@ -465,7 +472,7 @@ static void usage(void)
     fprintf(stderr, "    -R, --regions-file <file>     restrict to regions listed in a file\n");
     fprintf(stderr, "    -t, --targets <region>        similar to -r but streams rather than index-jumps\n");
     fprintf(stderr, "    -T, --targets-file <file>     similar to -R but streams rather than index-jumps\n");
-    fprintf(stderr, "        --threads <int>           number of extra output compression threads [0]\n");
+    fprintf(stderr, "        --threads <int>           use multithreading with <int> worker threads [0]\n");
     fprintf(stderr, "    -w, --write <list>            list of files to write with -p given as 1-based indexes. By default, all files are written\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Examples:\n");
@@ -478,6 +485,9 @@ static void usage(void)
     fprintf(stderr, "   # Extract and write records from A shared by both A and B using exact allele match\n");
     fprintf(stderr, "   bcftools isec A.vcf.gz B.vcf.gz -p dir -n =2 -w 1\n");
     fprintf(stderr, "\n");
+    fprintf(stderr, "   # Extract and write records from C found in A and C but not in B\n");
+    fprintf(stderr, "   bcftools isec A.vcf.gz B.vcf.gz C.vcf.gz -p dir -n~101 -w 3\n");
+    fprintf(stderr, "\n");
     fprintf(stderr, "   # Extract records private to A or B comparing by position only\n");
     fprintf(stderr, "   bcftools isec A.vcf.gz B.vcf.gz -p dir -n -1 -c all\n");
     fprintf(stderr, "\n");
@@ -540,7 +550,9 @@ int main_vcfisec(int argc, char *argv[])
                 else error("The --collapse string \"%s\" not recognised.\n", optarg);
                 break;
             case 'f': args->files->apply_filters = optarg; break;
-            case 'C': args->isec_op = OP_COMPLEMENT; break;
+            case 'C':
+                if ( args->isec_op!=0 && args->isec_op!=OP_COMPLEMENT ) error("Error: either -C or -n should be given, not both.\n");
+                args->isec_op = OP_COMPLEMENT; break;
             case 'r': args->regions_list = optarg; break;
             case 'R': args->regions_list = optarg; regions_is_file = 1; break;
             case 't': args->targets_list = optarg; break;
@@ -551,6 +563,8 @@ int main_vcfisec(int argc, char *argv[])
             case 'e': add_filter(args, optarg, FLT_EXCLUDE); break;
             case 'n':
                 {
+                    if ( args->isec_op!=0 && args->isec_op==OP_COMPLEMENT ) error("Error: either -C or -n should be given, not both.\n");
+                    if ( args->isec_op!=0 ) error("Error: -n should be given only once.\n");
                     char *p = optarg;
                     if ( *p=='-' ) { args->isec_op = OP_MINUS; p++; }
                     else if ( *p=='+' ) { args->isec_op = OP_PLUS; p++; }
@@ -565,7 +579,7 @@ int main_vcfisec(int argc, char *argv[])
             case  9 : args->n_threads = strtol(optarg, 0, 0); break;
             case  8 : args->record_cmd_line = 0; break;
             case 'h':
-            case '?': usage();
+            case '?': usage(); break;
             default: error("Unknown argument: %s\n", optarg);
         }
     }
