@@ -2,7 +2,7 @@
 
 /*  vcfroh.c -- HMM model for detecting runs of autozygosity.
 
-    Copyright (C) 2013-2018 Genome Research Ltd.
+    Copyright (C) 2013-2021 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -105,7 +105,7 @@ typedef struct _args_t
     int ntot;                   // some stats to detect if things didn't go wrong
     int nno_af;                 // number of sites rejected because AF could not be determined
     int nfiltered;              // .. because of filters
-    int nnot_biallelic, ndup;
+    int nno_alt, nmultiallelic, ndup;
     smpl_t *smpl;               // HMM data for each sample
     smpl_ilist_t *af_smpl;      // list of samples to estimate AF from (--estimate-AF)
     smpl_ilist_t *roh_smpl;     // list of samples to analyze (--samples, --samples-file)
@@ -113,6 +113,7 @@ typedef struct _args_t
     int af_from_PL;             // estimate AF from FMT/PL rather than FMT/GT
     char **argv, *targets_list, *regions_list, *af_fname, *af_tag, *samples, *buffer_size, *output_fname;
     int argc, fake_PLs, snps_only, vi_training, samples_is_file, output_type, skip_homref, n_threads;
+    int include_noalt_sites;
     BGZF *out;
     kstring_t str;
 
@@ -132,6 +133,11 @@ void *smalloc(size_t size)
     return mem;
 }
 
+static inline int max255(int i)
+{
+    return i < 256 ? i : 255;
+}
+
 static void init_data(args_t *args)
 {
     int i;
@@ -158,7 +164,7 @@ static void init_data(args_t *args)
         if ( !strncmp("GT,",args->estimate_AF,3) ) args->estimate_AF += 3;
         else if ( !strncmp("PL,",args->estimate_AF,3) ) { args->estimate_AF += 3; args->af_from_PL = 1; }
         if ( strcmp("-",args->estimate_AF) )
-            args->af_smpl = smpl_ilist_init(args->hdr, args->estimate_AF, 1, SMPL_NONE);
+            args->af_smpl = smpl_ilist_init(args->hdr, args->estimate_AF, 1, SMPL_NONE|SMPL_VERBOSE);
     }
 
     if ( args->estimate_AF || args->fake_PLs )
@@ -183,7 +189,7 @@ static void init_data(args_t *args)
             error("Error: The FORMAT/GT tag not found in the header\n");
     }
 
-    args->roh_smpl = smpl_ilist_init(args->hdr, args->samples, args->samples_is_file, SMPL_NONE);
+    args->roh_smpl = smpl_ilist_init(args->hdr, args->samples, args->samples_is_file, SMPL_NONE|SMPL_VERBOSE);
     if ( args->samples )
     {
         // we may be able to subset to a few samples, for a text VCF this can be a major speedup
@@ -545,6 +551,7 @@ static void flush_viterbi(args_t *args, int ismpl)
                     {
                         smpl->rg.state = 1;
                         smpl->rg.beg = smpl->sites[i];
+                        smpl->rg.end = smpl->sites[i];
                         smpl->rg.rid = args->prev_rid;
                         smpl->rg.qual  = qual;
                         smpl->rg.nqual = 1;
@@ -653,8 +660,10 @@ static void flush_viterbi(args_t *args, int ismpl)
     }
 }
 
-int read_AF(bcf_sr_regions_t *tgt, bcf1_t *line, double *alt_freq)
+int read_AF(args_t *args, bcf_sr_regions_t *tgt, bcf1_t *line, double *alt_freq)
 {
+    if ( tgt->nals < 2 )
+        error("Expected two comma-separated alleles (REF,ALT) in the third column of %s, found:\n\t%s\n", args->af_fname,tgt->line.s);
     if ( tgt->nals != line->n_allele ) return -1;    // number of alleles does not match
 
     int i;
@@ -751,9 +760,9 @@ int estimate_AF_from_PL(args_t *args, bcf_fmt_t *fmt_pl, int ial, double *alt_fr
                 if ( p[irr]<0 || p[ira]<0 || p[iaa]<0 ) continue;    /* missing value */ \
                 if ( p[irr]==p[ira] && p[irr]==p[iaa] ) continue;    /* all values are the same */ \
                 double prob[3], norm = 0; \
-                prob[0] = p[irr] < 256 ? args->pl2p[ p[irr] ] : args->pl2p[255]; \
-                prob[1] = p[ira] < 256 ? args->pl2p[ p[ira] ] : args->pl2p[255]; \
-                prob[2] = p[iaa] < 256 ? args->pl2p[ p[iaa] ] : args->pl2p[255]; \
+                prob[0] = args->pl2p[ max255(p[irr]) ]; \
+                prob[1] = args->pl2p[ max255(p[ira]) ]; \
+                prob[2] = args->pl2p[ max255(p[iaa]) ]; \
                 for (j=0; j<3; j++) norm += prob[j]; \
                 for (j=0; j<3; j++) prob[j] /= norm; \
                 af += 0.5*prob[1] + prob[2]; \
@@ -764,7 +773,7 @@ int estimate_AF_from_PL(args_t *args, bcf_fmt_t *fmt_pl, int ial, double *alt_fr
             case BCF_BT_INT8:  BRANCH(int8_t); break;
             case BCF_BT_INT16: BRANCH(int16_t); break;
             case BCF_BT_INT32: BRANCH(int32_t); break;
-            default: fprintf(bcftools_stderr,"Unknown format type for PL: %s:%d .. fmt->type=%d\n", __FILE__,__LINE__, fmt_pl->type); exit(1);
+            default: fprintf(bcftools_stderr,"Unknown format type for PL: %s:%d .. fmt->type=%d\n", __FILE__,__LINE__, fmt_pl->type); bcftools_exit(1);
         }
         #undef BRANCH
     }
@@ -781,9 +790,9 @@ int estimate_AF_from_PL(args_t *args, bcf_fmt_t *fmt_pl, int ial, double *alt_fr
                 if ( p[irr]<0 || p[ira]<0 || p[iaa]<0 ) continue;    /* missing value */ \
                 if ( p[irr]==p[ira] && p[irr]==p[iaa] ) continue;    /* all values are the same */ \
                 double prob[3], norm = 0; \
-                prob[0] = p[irr] < 256 ? args->pl2p[ p[irr] ] : args->pl2p[255]; \
-                prob[1] = p[ira] < 256 ? args->pl2p[ p[ira] ] : args->pl2p[255]; \
-                prob[2] = p[iaa] < 256 ? args->pl2p[ p[iaa] ] : args->pl2p[255]; \
+                prob[0] = args->pl2p[ max255(p[irr]) ]; \
+                prob[1] = args->pl2p[ max255(p[ira]) ]; \
+                prob[2] = args->pl2p[ max255(p[iaa]) ]; \
                 for (j=0; j<3; j++) norm += prob[j]; \
                 for (j=0; j<3; j++) prob[j] /= norm; \
                 af += 0.5*prob[1] + prob[2]; \
@@ -794,7 +803,7 @@ int estimate_AF_from_PL(args_t *args, bcf_fmt_t *fmt_pl, int ial, double *alt_fr
             case BCF_BT_INT8:  BRANCH(int8_t); break;
             case BCF_BT_INT16: BRANCH(int16_t); break;
             case BCF_BT_INT32: BRANCH(int32_t); break;
-            default: fprintf(bcftools_stderr,"Unknown format type for PL: %s:%d .. fmt->type=%d\n", __FILE__,__LINE__, fmt_pl->type); exit(1);
+            default: fprintf(bcftools_stderr,"Unknown format type for PL: %s:%d .. fmt->type=%d\n", __FILE__,__LINE__, fmt_pl->type); bcftools_exit(1);
         }
         #undef BRANCH
     }
@@ -829,12 +838,12 @@ int process_line(args_t *args, bcf1_t *line, int ial)
         if ( ret>0 )
             alt_freq = args->AFs[ial-1];
         if ( ret==-2 )
-            error("Type mismatch for INFO/%s tag at %s:%d\n", args->af_tag, bcf_seqname(args->hdr,line), line->pos+1);
+            error("Type mismatch for INFO/%s tag at %s:%"PRId64"\n", args->af_tag, bcf_seqname(args->hdr,line), (int64_t) line->pos+1);
     }
     else if ( args->af_fname ) 
     {
         // Read AF from a file
-        ret = read_AF(args->files->targets, line, &alt_freq);
+        ret = read_AF(args, args->files->targets, line, &alt_freq);
     }
     else if ( args->dflt_AF > 0 )
     {
@@ -928,15 +937,15 @@ int process_line(args_t *args, bcf1_t *line, int ial)
                 type_t *p = (type_t*)fmt_pl->p + fmt_pl->n*ismpl; \
                 if ( p[irr]<0 || p[ira]<0 || p[iaa]<0 ) continue;    /* missing value */ \
                 if ( p[irr]==p[ira] && p[irr]==p[iaa] ) continue;    /* all values are the same */ \
-                pdg[0] = p[irr] < 256 ? args->pl2p[ p[irr] ] : args->pl2p[255]; \
-                pdg[1] = p[ira] < 256 ? args->pl2p[ p[ira] ] : args->pl2p[255]; \
-                pdg[2] = p[iaa] < 256 ? args->pl2p[ p[iaa] ] : args->pl2p[255]; \
+                pdg[0] = args->pl2p[ max255(p[irr]) ]; \
+                pdg[1] = args->pl2p[ max255(p[ira]) ]; \
+                pdg[2] = args->pl2p[ max255(p[iaa]) ]; \
             }
             switch (fmt_pl->type) {
                 case BCF_BT_INT8:  BRANCH(int8_t); break;
                 case BCF_BT_INT16: BRANCH(int16_t); break;
                 case BCF_BT_INT32: BRANCH(int32_t); break;
-                default: fprintf(bcftools_stderr,"Unknown format type for PL: %s:%d .. fmt->type=%d\n", __FILE__,__LINE__, fmt_pl->type); exit(1);
+                default: fprintf(bcftools_stderr,"Unknown format type for PL: %s:%d .. fmt->type=%d\n", __FILE__,__LINE__, fmt_pl->type); bcftools_exit(1);
             }
             #undef BRANCH
         }
@@ -994,33 +1003,32 @@ static void vcfroh(args_t *args, bcf1_t *line)
 
     // Skip unwanted lines, for simplicity we consider only biallelic sites 
     if ( line->rid == args->skip_rid ) return;
-    if ( line->n_allele==1 ) { args->nnot_biallelic++; return; }   // no ALT allele
-    if ( line->n_allele > 3 ) { args->nnot_biallelic++; return; }   // cannot be bi-allelic, even with <*>
 
     // This can be raw callable VCF with the symbolic unseen allele <*>
-    int ial = 0;
+    int ial = 0, nalt = line->n_allele - 1;
     for (i=1; i<line->n_allele; i++)
-        if ( !strcmp("<*>",line->d.allele[i]) ) { ial = i; break; }
-    if ( ial==0 )    // normal VCF, the symbolic allele is not present
     {
-        if ( line->n_allele!=2 ) { args->nnot_biallelic++; return; }   // not biallelic
-        ial = 1;
+        if ( !strcmp("<*>",line->d.allele[i]) || !strcmp("<NON_REF>",line->d.allele[i]) ) nalt--;
+        else if ( !ial ) ial = i;
     }
-    else
+
+    if ( !nalt ) // no ALT allele
     {
-        if ( line->n_allele!=3 ) return;    // not biallelic
-        ial = ial==1 ? 2 : 1;               // <*> can come in any order
+        args->nno_alt++;
+        if ( !args->include_noalt_sites ) return;
     }
+    else if ( nalt>1 )
+    {
+        args->nmultiallelic++;
+        return;
+    }
+
     if ( args->snps_only && !bcf_is_snp(line) ) return;
 
     // Initialize genetic map
     int skip_rid = 0;
     if ( args->prev_rid<0 )
-    {
-        args->prev_rid = line->rid;
-        args->prev_pos = line->pos;
         skip_rid = load_genmap(args, bcf_seqname(args->hdr,line));
-    }
 
     // New chromosome?
     if ( args->prev_rid!=line->rid )
@@ -1068,7 +1076,7 @@ static void usage(args_t *args)
     fprintf(bcftools_stderr, "General Options:\n");
     fprintf(bcftools_stderr, "        --AF-dflt <float>              if AF is not known, use this allele frequency [skip]\n");
     fprintf(bcftools_stderr, "        --AF-tag <TAG>                 use TAG for allele frequency\n");
-    fprintf(bcftools_stderr, "        --AF-file <file>               read allele frequencies from file (CHR\\tPOS\\tREF\\tALT\\tAF)\n");
+    fprintf(bcftools_stderr, "        --AF-file <file>               read allele frequencies from file (CHR\\tPOS\\tREF,ALT\\tAF)\n");
     fprintf(bcftools_stderr, "    -b  --buffer-size <int[,int]>      buffer size and the number of overlapping sites, 0 for unlimited [0]\n");
     fprintf(bcftools_stderr, "                                           If the first number is negative, it is interpreted as the maximum memory to\n");
     fprintf(bcftools_stderr, "                                           use, in MB. The default overlap is set to roughly 1%% of the buffer size.\n");
@@ -1079,6 +1087,7 @@ static void usage(args_t *args)
     fprintf(bcftools_stderr, "                                           Safe value to use is 30 to account for GT errors.\n");
     fprintf(bcftools_stderr, "        --include <expr>               select sites for which the expression is true\n");
     fprintf(bcftools_stderr, "    -i, --ignore-homref                skip hom-ref genotypes (0/0)\n");
+    fprintf(bcftools_stderr, "        --include-noalt                include sites with no ALT allele (ignored by default)\n");
     fprintf(bcftools_stderr, "    -I, --skip-indels                  skip indels as their genotypes are enriched for errors\n");
     fprintf(bcftools_stderr, "    -m, --genetic-map <file>           genetic map in IMPUTE2 format, single file or mask, where string \"{CHROM}\"\n");
     fprintf(bcftools_stderr, "                                           is replaced with chromosome name\n");
@@ -1091,14 +1100,14 @@ static void usage(args_t *args)
     fprintf(bcftools_stderr, "    -S, --samples-file <file>          file of samples to analyze [all samples]\n");
     fprintf(bcftools_stderr, "    -t, --targets <region>             similar to -r but streams rather than index-jumps\n");
     fprintf(bcftools_stderr, "    -T, --targets-file <file>          similar to -R but streams rather than index-jumps\n");
-    fprintf(bcftools_stderr, "        --threads <int>                number of extra decompression threads [0]\n");
+    fprintf(bcftools_stderr, "        --threads <int>                use multithreading with <int> worker threads [0]\n");
     fprintf(bcftools_stderr, "\n");
     fprintf(bcftools_stderr, "HMM Options:\n");
     fprintf(bcftools_stderr, "    -a, --hw-to-az <float>             P(AZ|HW) transition probability from HW (Hardy-Weinberg) to AZ (autozygous) state [6.7e-8]\n");
     fprintf(bcftools_stderr, "    -H, --az-to-hw <float>             P(HW|AZ) transition probability from AZ to HW state [5e-9]\n");
     fprintf(bcftools_stderr, "    -V, --viterbi-training <float>     estimate HMM parameters, <float> is the convergence threshold, e.g. 1e-10 (experimental)\n");
     fprintf(bcftools_stderr, "\n");
-    exit(1);
+    bcftools_exit(1);
 }
 
 int main_vcfroh(int argc, char *argv[])
@@ -1119,6 +1128,7 @@ int main_vcfroh(int argc, char *argv[])
         {"AF-dflt",1,0,2},
         {"include",1,0,3},
         {"exclude",1,0,4},
+        {"include-noalt",0,0,5},
         {"buffer-size",1,0,'b'},
         {"ignore-homref",0,0,'i'},
         {"estimate-AF",1,0,'e'},
@@ -1151,8 +1161,13 @@ int main_vcfroh(int argc, char *argv[])
                 args->dflt_AF = strtod(optarg,&tmp);
                 if ( *tmp ) error("Could not parse: --AF-dflt %s\n", optarg);
                 break;
-            case 3: args->filter_str = optarg; args->filter_logic = FLT_INCLUDE; break;
-            case 4: args->filter_str = optarg; args->filter_logic = FLT_EXCLUDE; break;
+            case  3 :
+                if ( args->filter_str ) error("Error: only one --include or --exclude expression can be given, and they cannot be combined\n");
+                args->filter_str = optarg; args->filter_logic |= FLT_INCLUDE; break;
+            case  4 :
+                if ( args->filter_str ) error("Error: only one --include or --exclude expression can be given, and they cannot be combined\n");
+                args->filter_str = optarg; args->filter_logic |= FLT_EXCLUDE; break;
+            case 5: args->include_noalt_sites = 1; break;
             case 'o': args->output_fname = optarg; break;
             case 'O': 
                 if ( strchr(optarg,'s') || strchr(optarg,'S') ) args->output_type |= OUTPUT_ST;
@@ -1200,7 +1215,7 @@ int main_vcfroh(int argc, char *argv[])
         }
     }
     if ( !args->output_fname ) args->output_fname = "bcftools_stdout";
-    if ( !args->output_type ) args->output_type = OUTPUT_ST|OUTPUT_RG;
+    if ( !args->output_type || args->output_type==OUTPUT_GZ ) args->output_type |= OUTPUT_ST|OUTPUT_RG;
     char *fname = NULL;
     if ( optind==argc )
     {
@@ -1231,7 +1246,7 @@ int main_vcfroh(int argc, char *argv[])
     }
     if ( args->n_threads && bcf_sr_set_threads(args->files, args->n_threads)<0)
         error("Failed to create threads\n");
-    if ( !bcf_sr_add_reader(args->files, fname) ) error("Failed to open %s: %s\n", fname,bcf_sr_strerror(args->files->errnum));
+    if ( !bcf_sr_add_reader(args->files, fname) ) error("Failed to read from %s: %s\n", !strcmp("-",fname)?"standard input":fname,bcf_sr_strerror(args->files->errnum));
 
     init_data(args);
     while ( bcf_sr_next_line(args->files) )
@@ -1254,7 +1269,7 @@ int main_vcfroh(int argc, char *argv[])
         fprintf(bcftools_stderr,"Number of lines overlapping with --AF-file/processed: %d/%d\n", args->ntot,nmin);
     else
         fprintf(bcftools_stderr,"Number of lines total/processed: %d/%d\n", args->ntot,nmin);
-    fprintf(bcftools_stderr,"Number of lines filtered/no AF/not biallelic/dup: %d/%d/%d/%d\n", args->nfiltered,args->nno_af,args->nnot_biallelic,args->ndup);
+    fprintf(bcftools_stderr,"Number of lines filtered/no AF/no alt/multiallelic/dup: %d/%d/%d/%d/%d\n", args->nfiltered,args->nno_af,args->nno_alt,args->nmultiallelic,args->ndup);
     if ( nmin==0 )
     {
         fprintf(bcftools_stderr,"No usable sites were found.\n");
