@@ -4,18 +4,22 @@ BCFTOOLS_DISPATCH = [
     "index",
     "annotate",
     "concat",
+    "convert",
     "isec",
     "merge",
     "norm",
     "plugin",
     "query",
     "reheader",
+    "sort",
     "view",
     "call",
     "consensus",
     "cnv",
+    "csq",
     "filter",
     "gtcheck",
+    "mpileup",
     "roh",
     "stats"]
 
