@@ -1,8 +1,6 @@
 from pysam.utils import PysamDispatcher
 
 # samtools command line options to export in python
-#
-# import is a python reserved word.
 SAMTOOLS_DISPATCH = {
     # samtools 'documented' commands
     "view": ("view", None),
@@ -10,6 +8,7 @@ SAMTOOLS_DISPATCH = {
     "mpileup": ("mpileup", None),
     "depth": ("depth", None),
     "faidx": ("faidx", None),
+    "fqidx": ("fqidx", None),
     "tview": ("tview", None),
     "index": ("index", None),
     "idxstats": ("idxstats", None),
@@ -17,18 +16,19 @@ SAMTOOLS_DISPATCH = {
     "flagstat": ("flagstat", None),
     "calmd": ("calmd", None),
     "merge": ("merge", None),
+    "markdup": ("markdup", None),
     "rmdup": ("rmdup", None),
     "reheader": ("reheader", None),
     "cat": ("cat", None),
     "targetcut": ("targetcut", None),
     "phase": ("phase", None),
-    "samimport": ("import", None),
     "bam2fq": ("bam2fq", None),
     "dict": ("dict", None),
     "addreplacerg": ("addreplacerg", None),
     "pad2unpad": ("pad2unpad", None),
     "depad": ("pad2unpad", None),
     "bedcov": ("bedcov", None),
+    "coverage": ("coverage", None),
     "bamshuf": ("bamshuf", None),
     "collate": ("collate", None),
     "stats": ("stats", None),
@@ -36,6 +36,11 @@ SAMTOOLS_DISPATCH = {
     "fastq": ("fastq", None),
     "quickcheck": ("quickcheck", None),
     "split": ("split", None),
+    "flags": ("flags", None),
+    "ampliconclip": ("ampliconclip", None),
+    "ampliconstats": ("ampliconstats", None),
+    "version": ("version", None),
+    "fqimport": ("import", None),
 }
 
 # instantiate samtools commands as python functions
