@@ -7,7 +7,7 @@ import string
 import copy
 import array
 
-from TestUtils import checkFieldEqual, BAM_DATADIR, get_temp_filename, get_temp_context, IS_PYTHON3
+from TestUtils import checkFieldEqual, make_data_files, BAM_DATADIR, get_temp_filename, get_temp_context, IS_PYTHON3
 
 
 if IS_PYTHON3:
@@ -15,6 +15,11 @@ if IS_PYTHON3:
 else:
     maketrans = string.maketrans
 
+
+def setUpModule():
+    make_data_files(BAM_DATADIR)
+
+
 class ReadTest(unittest.TestCase):
 
     def build_read(self):
@@ -65,7 +70,7 @@ class TestAlignedSegment(ReadTest):
         a = pysam.AlignedSegment()
         s = str(a)
         self.assertEqual(
-            "None\t0\t-1\t-1\t0\tNone\t-1\t-1\t0\tNone\tNone\t[]",
+            "None\t0\t*\t0\t0\tNone\t*\t0\t0\tNone\tNone\t[]",
             s)
 
     def testSettingTagInEmptyRead(self):
@@ -525,13 +530,13 @@ class TestAlignedSegment(ReadTest):
     def test_query_length_is_limited(self):
         a = self.build_read()
         a.query_name = "A" * 1
-        a.query_name = "A" * 251
+        a.query_name = "A" * 254
         self.assertRaises(
             ValueError,
             setattr,
             a,
             "query_name",
-            "A" * 252)
+            "A" * 255)
 
     def test_header_accessible(self):
         a = self.build_read()
