@@ -24,7 +24,11 @@ import pysam
 import pysam.samtools
 from TestUtils import checkBinaryEqual, checkGZBinaryEqual, check_url, \
     check_samtools_view_equal, checkFieldEqual, force_str, \
-    get_temp_filename, BAM_DATADIR
+    get_temp_filename, make_data_files, BAM_DATADIR
+
+
+def setUpModule():
+    make_data_files(BAM_DATADIR)
 
 
 ##################################################
@@ -177,9 +181,9 @@ class BasicTestBAMFromFetch(unittest.TestCase):
 
     def testPresentOptionalFields(self):
         self.assertEqual(
-            self.reads[0].opt('NM'), 1,
+            self.reads[0].opt('NM'), 22,
             "optional field mismatch in read 1, NM: %s != %s" %
-            (self.reads[0].opt('NM'), 1))
+            (self.reads[0].opt('NM'), 22))
         self.assertEqual(
             self.reads[0].opt('RG'), 'L1',
             "optional field mismatch in read 1, RG: %s != %s" %
@@ -205,7 +209,7 @@ class BasicTestBAMFromFetch(unittest.TestCase):
 
     def testTags(self):
         self.assertEqual(self.reads[0].tags,
-                         [('NM', 1), ('RG', 'L1'),
+                         [('NM', 22), ('RG', 'L1'),
                           ('PG', 'P1'), ('XT', 'U')])
         self.assertEqual(self.reads[1].tags,
                          [('MF', 18), ('RG', 'L2'),
@@ -723,7 +727,7 @@ class TestIO(unittest.TestCase):
         read = load_bam()
         self.assertEqual(read.reference_name, "chr1")
         
-    # TOOD
+    # TODO
     # def testReadingFromSamFileWithoutHeader(self):
     #     '''read from samfile without header.
     #     '''
@@ -1382,21 +1386,6 @@ class TestDeNovoConstruction(unittest.TestCase):
         os.unlink(tmpfilename)
 
 
-class TestDeNovoConstructionUserTags(TestDeNovoConstruction):
-
-    '''test de novo construction with a header that contains lower-case tags.'''
-
-    header = {'HD': {'VN': '1.0'},
-              'SQ': [{'LN': 1575, 'SN': 'chr1'},
-                     {'LN': 1584, 'SN': 'chr2'}],
-              'x1': {'A': 2, 'B': 5},
-              'x3': {'A': 6, 'B': 5},
-              'x2': {'A': 4, 'B': 5}}
-
-    bamfile = os.path.join(BAM_DATADIR, "example_user_header.bam")
-    samfile = os.path.join(BAM_DATADIR, "example_user_header.sam")
-
-
 class TestEmptyHeader(unittest.TestCase):
     '''see issue 84.'''
 
@@ -1406,12 +1395,12 @@ class TestEmptyHeader(unittest.TestCase):
         self.assertEqual(s.header.to_dict(), {'SQ': [{'LN': 1000, 'SN': 'chr1'}]})
 
     def test_bam_without_seq_in_header(self):
-        s = pysam.AlignmentFile(os.path.join(BAM_DATADIR, "example_no_seq_in_header.bam"))
+        s = pysam.AlignmentFile(os.path.join(BAM_DATADIR, "0example_no_seq_in_header.bam"))
         self.assertTrue("SQ" in s.header.to_dict())
         self.assertTrue("@SQ" in str(s.header))
 
     def test_bam_without_seq_with_null_bytes_in_header(self):
-        s = pysam.AlignmentFile(os.path.join(BAM_DATADIR, "example_no_seq_in_header_null_bytes.bam"))
+        s = pysam.AlignmentFile(os.path.join(BAM_DATADIR, "0example_no_seq_in_header_null_bytes.bam"))
         self.assertTrue("SQ" in s.header.to_dict())
         self.assertTrue("@SQ" in str(s.header))
 
@@ -1454,7 +1443,7 @@ class TestHeaderWithProgramOptions(unittest.TestCase):
                      'ID': 'bwa',
                      'VN': '0.7.9a-r786',
                      'CL': 'bwa mem -p -t 8 -M -R '
-                     '@RG\tID:None\tSM:None\t/mnt/data/hg19.fa\t'
+                     '@RG ID:None SM:None /mnt/data/hg19.fa '
                      '/mnt/analysis/default-0.fastq'}]})
 
 
@@ -1475,6 +1464,24 @@ class TestTruncatedBAM(unittest.TestCase):
             return len([a for a in x])
         self.assertRaises(IOError, iterall, s)
 
+        # Ignore closing errors, as s is now in an error state
+        try:
+            s.close()
+        except IOError:
+            pass
+
+
+class TestCorruptBAM(unittest.TestCase):
+    """See pull request 1035."""
+
+    def testCorruptBamIterator(self):
+        s = pysam.AlignmentFile(os.path.join(BAM_DATADIR, "ex2_corrupt.bam"))
+
+        def iterall(x):
+            return len([a for a in x])
+
+        self.assertRaises(IOError, iterall, s)
+
 
 COMPARE_BTAG = [100, 1, 91, 0, 7, 101, 0, 201, 96, 204,
                 0, 0, 87, 109, 0, 7, 97, 112, 1, 12, 78,
@@ -2051,6 +2058,10 @@ class TestLogging(unittest.TestCase):
             cols = bam.pileup()
         self.assertTrue(True)
 
+    def tearDown(self):
+        if os.path.exists('log.txt'):
+            os.unlink('log.txt')
+
     def testFail1(self):
         self.check(os.path.join(BAM_DATADIR, "ex9_fail.bam"),
                    False)
@@ -2327,21 +2338,6 @@ class TestSanityCheckingBAM(unittest.TestCase):
         self.check_write(read)
 
 
-class TestHeader1000Genomes(unittest.TestCase):
-
-    '''see issue 110'''
-    bamfile = "http://ftp.1000genomes.ebi.ac.uk/vol1/ftp/technical/phase3_EX_or_LC_only_alignment/data/HG00104/alignment/HG00104.chrom11.ILLUMINA.bwa.GBR.low_coverage.20130415.bam"  # noqa
-
-    def testRead(self):
-
-        if not check_url(self.bamfile):
-            return
-
-        f = pysam.AlignmentFile(self.bamfile, "rb")
-        data = f.header.copy()
-        self.assertTrue(data)
-
-
 class TestLargeCigar(unittest.TestCase):
 
     def setUp(self):
@@ -2428,9 +2424,6 @@ class TestLargeCigar(unittest.TestCase):
 #     mode = "w"
 
 if __name__ == "__main__":
-    # build data files
-    print("building data files")
-    subprocess.call("make -C %s" % BAM_DATADIR, shell=True)
     print("starting tests")
     unittest.main()
     print("completed tests")
