@@ -5,6 +5,7 @@ import difflib
 import gzip
 import contextlib
 import inspect
+import subprocess
 import tempfile
 import pysam
 
@@ -251,6 +252,18 @@ def get_temp_context(suffix="", keep=False):
             os.unlink(f)
 
 
+def make_data_files(directory):
+    what = None
+    try:
+        if not os.path.exists(os.path.join(directory, "all.stamp")):
+            subprocess.check_output(["make", "-C", directory], stderr=subprocess.STDOUT)
+    except subprocess.CalledProcessError as e:
+        what = "Making test data in '%s' failed:\n%s" % (directory, force_str(e.output))
+
+    if what is not None:
+        raise RuntimeError(what)
+
+
 def load_and_convert(filename, encode=True):
     '''load data from filename and convert all fields to string.
 
