@@ -7,14 +7,17 @@ import unittest
 import pysam
 import shutil
 import gzip
-import subprocess
 
 try:
     from pathlib import Path
 except ImportError:
     Path = None
 
-from TestUtils import get_temp_filename, check_lines_equal, load_and_convert, CBCF_DATADIR, get_temp_context
+from TestUtils import get_temp_filename, check_lines_equal, load_and_convert, make_data_files, CBCF_DATADIR, get_temp_context
+
+
+def setUpModule():
+    make_data_files(CBCF_DATADIR)
 
 
 def read_header(filename):
@@ -33,6 +36,12 @@ def read_header(filename):
     return data
 
 
+def read_index_header(filename):
+    with gzip.open(filename) as infile:
+        magic = infile.read(4)
+    return magic
+
+
 class TestMissingGenotypes(unittest.TestCase):
 
     filename = "missing_genotypes.vcf"
@@ -199,6 +208,7 @@ class TestIndexFormatsVCF(unittest.TestCase):
             shutil.copyfile(self.vcf_filename, fn)
             pysam.tabix_index(fn, preset="vcf", force=True)
             self.assertTrue(os.path.exists(fn + ".gz" + ".tbi"))
+            self.assertEqual(read_index_header(fn + ".gz.tbi"), b"TBI\1")
             self.assertFalse(os.path.exists(fn + ".gz" + ".csi"))
             
             with pysam.VariantFile(fn + ".gz") as inf:
@@ -210,6 +220,7 @@ class TestIndexFormatsVCF(unittest.TestCase):
 
             pysam.tabix_index(fn, preset="vcf", force=True, csi=True)
             self.assertTrue(os.path.exists(fn + ".gz" + ".csi"))
+            self.assertEqual(read_index_header(fn + ".gz.csi"), b"CSI\1")
             self.assertFalse(os.path.exists(fn + ".gz" + ".tbi"))
             
             with pysam.VariantFile(fn + ".gz") as inf:
@@ -221,6 +232,7 @@ class TestIndexFormatsVCF(unittest.TestCase):
             shutil.copyfile(self.bcf_filename + ".csi", fn + ".csi")
 
             self.assertTrue(os.path.exists(fn + ".csi"))
+            self.assertEqual(read_index_header(fn + ".csi"), b"CSI\1")
             self.assertFalse(os.path.exists(fn + ".tbi"))
             
             with pysam.VariantFile(fn) as inf:
@@ -232,6 +244,7 @@ class TestIndexFormatsVCF(unittest.TestCase):
 
             pysam.tabix_index(fn, preset="bcf", force=True, csi=False)
             self.assertTrue(os.path.exists(fn + ".csi"))
+            self.assertEqual(read_index_header(fn + ".csi"), b"CSI\1")
             self.assertFalse(os.path.exists(fn + ".tbi"))
             
             with pysam.VariantFile(fn) as inf:
@@ -244,6 +257,7 @@ class TestIndexFormatsVCF(unittest.TestCase):
             pysam.tabix_index(fn, preset="vcf", force=True, csi=True)
             
             self.assertTrue(os.path.exists(fn + ".csi"))
+            self.assertEqual(read_index_header(fn + ".csi"), b"CSI\1")
             self.assertFalse(os.path.exists(fn + ".tbi"))
             
             with pysam.VariantFile(fn) as inf:
@@ -668,9 +682,6 @@ class TestUnicode(unittest.TestCase):
                 
 
 if __name__ == "__main__":
-    # build data files
-    print("building data files")
-    subprocess.call("make -C %s" % CBCF_DATADIR, shell=True)
     print("starting tests")
     unittest.main()
     print("completed tests")
